# This file was auto-generated by Fern from our API Definition.

import typing

IntegrationName = typing.Union[
    typing.Literal[
        "SLACK",
        "NOTION",
        "GOOGLE",
        "CALENDLY",
        "HUBSPOT",
        "LINEAR",
        "GITHUB",
        "GOOGLE_SHEETS",
        "GOOGLE_CALENDAR",
        "GOOGLE_DRIVE",
        "GMAIL",
        "AIRTABLE",
    ],
    typing.Any,
]
