# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .vellum_error import VellumError


class TestSuiteRunExecutionErrorOutput(UniversalBaseModel):
    """
    Execution output of an entity evaluated during a Test Suite Run that is of type ERROR
    """

    name: str
    type: typing.Literal["ERROR"] = "ERROR"
    value: typing.Optional[VellumError] = None
    output_variable_id: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
